#include <stdint.h>
#include "resources.h"

/*
 * TFT pins
 ******************************************************************************/
char TFT_DataPort           	at LATE;
sbit TFT_RST                	at LATD7_bit;
sbit TFT_BLED               	at LATD2_bit;
sbit TFT_RS                 	at LATD9_bit;
sbit TFT_CS                 	at LATD10_bit;
sbit TFT_RD                 	at LATD5_bit;
sbit TFT_WR                 	at LATD4_bit;
char TFT_DataPort_Direction 	at TRISE;
sbit TFT_RST_Direction      	at TRISD7_bit;
sbit TFT_BLED_Direction     	at TRISD2_bit;
sbit TFT_RS_Direction       	at TRISD9_bit;
sbit TFT_CS_Direction       	at TRISD10_bit;
sbit TFT_RD_Direction       	at TRISD5_bit;
sbit TFT_WR_Direction       	at TRISD4_bit;

/*
 * Ozone CS pin
 ******************************************************************************/
sbit OZONE_CS at LATC2_bit;
sbit OZONE_CS_DIR at TRISC2_bit;

/*
 * Global vars
 ******************************************************************************/
const float    Rr    = 10000.0;
const float    Vref  = 5.0;
uint16_t       raw;
float          ozone;
char           text[ 20 ];

/*
 * Prototypes
 ******************************************************************************/
void system_init( void );
void display_init( void );
void display_result( void );
void measurement( void );
void calculation( void );

/*
 * Functions
 ******************************************************************************/
void system_init()
{
    OZONE_CS_DIR = 0;
    Delay_ms( 200 );

    SPI3_Init_Advanced( _SPI_MASTER, _SPI_8_BIT, 1, _SPI_SS_DISABLE,
                        _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH,
                        _SPI_ACTIVE_2_IDLE );
    Delay_ms( 200 );

    OZONE_CS = 1;
}

void display_init()
{
    TFT_Init_ILI9341_8bit( 320, 240 );
    TFT_BLED = 1;
    TFT_Set_Pen( CL_WHITE, 1 );
    TFT_Set_Brush( 1, CL_WHITE, 0, 0, 0, 0 );
    TFT_Set_Font( TFT_defaultFont, CL_BLACK, FO_HORIZONTAL );
    TFT_Fill_Screen( CL_WHITE );
    TFT_Set_Pen( CL_BLACK, 1 );
    TFT_Line(  20,  46, 300,  46 );
    TFT_Line(  20,  70, 300,  70 );
    TFT_Line(  20, 220, 300, 220 );
    TFT_Set_Pen( CL_WHITE, 1 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "Ozone click", 110, 14 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "Ozone ( PPB ) :", 100, 110 );
    TFT_Set_Font( &Tahoma15x16_Bold, CL_BLUE, FO_HORIZONTAL );
    TFT_Write_Text( "MiCS-2614", 125, 50 );
    TFT_Set_Font( &Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "EasyMx PRO v7 for STM32", 19, 223 );
    TFT_Set_Font( &Verdana12x13_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "www.mikroe.com", 200, 223 );
    TFT_Set_Font( &Tahoma15x16_Bold, CL_RED, FO_HORIZONTAL );
    Delay_ms( 200 );
}

void display_result()
{
    FloatToStr( ozone, text );
    TFT_Rectangle( 100, 145, 300, 170 );
    TFT_Write_Text( text, 128, 145 );
}

void measurement()
{
    uint8_t msb;
    uint8_t lsb;

    OZONE_CS = 0;
    msb = SPI_Rd_Ptr( 0xFF );
    lsb = SPI_Rd_Ptr( 0xFF );
    OZONE_CS = 1;

    raw = ( msb & 0x1F );
    raw <<= 7;
    raw |= ( lsb >> 1 );
}

void calculation()
{
    float Vout;
    float Rs;
    float ratio;
    float lgPPM;

    Vout    = ( float )raw / 4096.0 * Vref;
    FloatToStr( Vout, text );
    Rs = Rr / ( Vref - Vout ) * Vout;
    FloatToStr( Rs, text );
    ratio   = Rs / Rr;
    ozone   = ( ( log10( ratio ) *  -0.8 ) + 0.9 ) * 1000;  // Parts per billion
}

/*
 * Main
 ******************************************************************************/
void main()
{
    system_init();
    display_init();

    while( 1 )
    {
        measurement();
        calculation();
        display_result();
        Delay_ms( 500 );
    }
}